CosinorSpan_PeriVsCosFiltCATmulti3varsProg <-
function (TimeCol=1,Y=2, Components=1, window="noFilt", RefDateTime="0",  timeFormat="%Y%m%d%H%M", RangeDateTime=list(Start=0, End=0), Units='Hour', dt=0, Progressive=list(Span=144, Increment=8), Period=list(Set=0,Start=0,Increment=1,End=0),header=F, Skip=0, Colors="BW",Graphics="pdf",Output=list(Txt=F,TxtAmp=F,Doc=T,HeatMap=T,LineGraphs=F),Console=F,Debug=FALSE,call=FALSE) {
  # Header:  T/F to indicate if the file has a header
  #%H represents 0-23, so have to have numeric time in Excel sheet, and numeric time rep option
  # Dates are represented as the number of days since 1970-01-01
  #strptime reads in characters and produces POSIX formatted date/time;  strptime accepts the format of date being read -- output is always POSIX
  #use format wrapped around POSIX to output a date in a different format than POSIX
  #POSIXct: This class stores a date in a vector, as the number of seconds passed since January, 1st, 1970 at 1AM
  #POSIXlt: This class stores a date in a list: (day of the month, day of the week, day of the year, month, year, time zone, hour, minute, second).
  # TimeCol:  Column where time is found
  # Y:  Data to be analyzed
  # Components:  Default=1.  Indicates if this is a single or multiple component cosinor analysis, where the number of components is specified (>0).  
  # window:      Data will be convolved with a window:  hanning, hamming, bartlett, blackman.  Default="noFilt"
  # RefDateTime:  Date used as reference, and subtracted from all data dates, to make the number smaller.  **Must be in the same time zone!!!!!**
  #             if RefDateTime = 0, use the 1st date of the data as the RefDateTime 
  #             if RefDateTime = 12, use midnight of the same day as the data starts 
  # timeFormat:   RefDateTime and the time in the data file should both use the same date/time format.   Can be "numeric" or "%Y%m%d%H%M"
  #             i.e., "%Y%m%d%H%M", or if = "numeric", time column in data file can be simple numbers (0 - 99999...)
  #              ToDo:  modify to use alternate time formats
  # RangeDateTime
  #      $Start:  Analysis is started at this date/time.  May be before 1st data date.
  #             if Start = 0, use the 1st time in the data set
  #             if Start = 12 use Midnight at the start of the 1st date of the data as the StartDate 
  #        $End:    Analysis ends at this date/time.  May be after last data date.
  #             if End = 0, use the last time in the data set
  #             if End = 12 use the midnight at the end of the last date of the data as the EndDate   
  # Units:        Units (Hour, Year, Week or Day) for Span and Increment arguments, as well as Period arguments
  #               ToDo:  modify to use units other than Hours
  # Progressive
  #      $Span : number of Days, Wks or Yrs in a subset of full data  -- multiple spans calculated
  #               If 0, assumes no progession, Span is set to the full dataset length, and Increment = full data set  
  #  $Increment: number of Days, Wks or Yrs  (uses same unit as set for Span) to shift between span analyses
  #               If 0, assumes no progession, Span is set to the full dataset length, and Increment = full data set 
  # Period
  #       $End : Last (and smallest) period to calculate, in units of Days, Wks or Yrs (as set by Units)
  #               Defaults to 2;  (1 is too small) 0 is invalid  -- says use default
  #  $Increment : Increment to starting period, in units of Days, Wks or Yrs (as set by Units)
  #               Defaults to 1;   0 is invalid  -- says use default
  #     $Start : First (and largest) period to calculate, in units of Days, Wks or Yrs (as set by Units);  (Span/1)
  #               0 is Default: full time range analyzed [in hours?] (RangeDateTime$End-RangeDateTime$Start)= MyData_length; or if progressive, Span/1;     
  #      $Set :  Overrides Period$Start and $Increment, to specify a specific set of periods to analyze (as set by Units).  If $End will determine ending period 
  #              Can be in the format of a single number, or an array:  c(1,2,3) or c(8,12:24)
  #              When Components=1, each period specified in the vector will be assessed by cosinor independently.
  #              When parameter Components is >1, Period$Set must have a correspondig number of components, which assessed together in a multiple component cosinor.
  #              If 0, a series of periods are analyzed (spectrum) according to Period$Start, $Increment, $End (or their default value, if not specified)
  #              When 0, only the maximum period, or the maximum period per span, from a spectrum is displayed on the graphics file.
  #              Otherwise, all periods are displayed on the graphic 
  #  Colors       "Heat" renders the heat map in colors;  "BW" renders the heatmap in grayscale
  #  Graphics    Possible values:  jpg, pdf, tif, png
  #  Output      Txt will generate a comma delimited .txt document containing output,Doc will generate a nicely formatted Word doc
  #                  HeatMap will generate a heatmap, but only when more than one period is specified for a progressive analysis
  #                 LineGraphs will generate a set of graphs plotting MESOR, Amplitude, Acrophase over time--too many when you do a progressive
  # dt           When equidistant data, dt indicates the sampling interval.  If dt =0, no periodogram is done.
  #                  Data is assumed to be equidistant when this is nonzero
#   source('~/Documents/Cathy/Neuroscience papers/capstone/Bartolomucci/chrono data/R-functions/CatPeri/CatPeri.R')    #  Add for MAC
#   source('~/Documents/Cathy/Neuroscience papers/capstone/Chronobiology/CosinorR/Cosinor-Hamming-sine/CatWindow.R')
#   source('~/Documents/Cathy/Neuroscience papers/capstone/Bartolomucci/chrono data/R-functions/CatBin/CatBin.R')
#   source('~/Documents/Cathy/Neuroscience papers/capstone/Chronobiology/CosinorR/ANOVA/CatPlexi.R')
  #source('E:/R/Cosinor May1-2013/periCosinor.R')     # Add for PC
  #library(season)
  functionName<-"CosinorSpan_PeriVsCosFiltCATmulti3varsProg"
  tz="GMT"
  
  if (call==TRUE){    #  chg TimeCol 1, dt 2/60, Y 2, timeFormat<-"%Y/%m/%d %H:%M:%S", Components ?, line 210 less numerics    
                      #  or chg TimeCol 2, dt 0, Y 4, timeFormat<-"%Y%m%d%H%M" line 210 add numerics 
    header<-F          #These lines are needed to set variables when not run as a function
    TimeCol<-1          #c(1,2)    ##### for mouse data
    Components<-3
    dt<-(0)
    Y<-4           #### change for mouse data  -- use exact column number from file 
    Skip=0
    RefDateTime<- "196712240000"              #"200910180000"    for mine: "201302030000"    "12"  "0"  
    RangeDateTime<-list(Start="0", End="0")
    timeFormat<-"%Y%m%d%H%M"      #"%Y/%m/%d %H:%M:%S"                        #numeric"
    Units<-'Hour'              #  Hour, Day, Week, Year
    Plex<-TRUE
    foldLen<-24
    Progressive<-list(Span=35064, Increment=1848)            #  Progressive spans of time   144,8   1807.5
    Period <-list(Set=c(8766,4383,3615),Start=0,Increment=1,End=0)     #  format option:  Set=c(24,26,8)   
    Debug<-FALSE
    Colors="BW"       #"terrain"                  #Heat"   #BW
    #terrain.colors(n, alpha = 1)
    #topo.colors(n, alpha = 1)
    Output=list(Txt=F,TxtAmp=F,Doc=T,HeatMap=F,LineGraphs=F)
    
    Graphics<-"pdf"
    Console<-F
    
    #timeFormat<-"%Y%m%d%H%M"
    #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Chronobiology/CosinorR/Data/clgi001.dat'}  #  Add for MAC
    #fileName<-'E:\\R\\CLGi001.pr0'}            # Add for PC
    #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Chronobiology/CosinorR/Data/FH3h1wk.dat.txt'}
    #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Chronobiology/CosinorR/Data/Signal10-20.txt'}
    #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Testing/multiple/activity stress c57--stress copy2-short.txt'}
    #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Testing/multiple/CG140129.pr0'}
    #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Testing/multiple/testcasebliss/BlissT17-1.dat.txt'}
  
    fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Testing/multiple/testcase3varsProg/3vars.dat.txt'}
    #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Testing/FWtestN15/sinesnoneN500--09Oct13--14-03-58.txt'}
  #fileName<-'~/Documents/ChronobiologyLab/4Cathy/NAIN20080320.dta'}
  #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Testing/FWtestN15/sinesnoneN15--07Oct13--16-58-55.txt'}
  #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Testing/FWtestN15/FWtestN15missing/sinesnoneN15--07Oct13--16-58-55-miss.txt'}
  ##fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Testing/FWtestN0/FWtestN0-missing/sinesnoneN0--07Oct13--15-26-38-miss.txt'}   
  #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Testing/FWtestN0/sinesnoneN0--07Oct13--15-26-38.txt'}
    #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Chronobiology/CosinorR/Data/HanningSignal10-20.txt'}
    #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Bartolomucci/Data/Good/July03-2013/good-6d-2m-part.txt'}
    #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Bartolomucci/Data/Good/July03-2013/activity stress c57--stress copy2-part.txt'}
    #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Chronobiology/CosinorR/results/reference/FW1/FWedited.txt'}
    #  jaggy  fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Chronobiology/CosinorR/Cosinor-Hamming-sine/sinesnone--27Aug13--16-34-43.txt'}
    #fileName<-'~/Documents/Cathy/Neuroscience papers/capstone/Chronobiology/CosinorR/Cosinor-Hamming-sine/sineshanning-cos---25Aug13--23-19-37'}
  else {
    # read data
    fileName <- file.choose()
  }
  if (Components>1){
  "%gcd%" <- function(u, v) {ifelse(u %% v != 0, v %gcd% (u%%v), v)}
  "%lcm%" <- function(u, v) { abs(u*v)/(u %gcd% v)}
  LCM<-1
    for (i in 1:Components){
      LCM<-Period$Set[i] %lcm% LCM
    }
  }
  # rename parameters
  StartDate<-RangeDateTime$Start
  EndDate<-RangeDateTime$End
  Span<-Progressive$Span
  Increment<-Progressive$Increment
  FreqInc<-Period$Increment
  FreqStart<-Period$Start
  FreqEnd<-Period$End
  oneCycle<-Period$Set     # can be a vector of one or more
  Ys_end<-length(Y)
  CosMatrixDim<-Components*2+1         #  validate and error if too large -- should it be Components*2?  no Feb 26
  if (oneCycle[1]>0){
    if (Components>1 && length(oneCycle)!=Components){
      stop(paste("Error:  When parameter Components is >1, Period$Set must have a correspondig number of components.  \nComponents=",Components,"; Period$Set=",Period$Set))
       # Period$Set will contain multiple possible components to check separately, or as a multi-component cosinor, depending on param Components
       # if Components=1, Period$Set can have any number of components.  If Components>1, length(oneCyce) must equal Components
    }
  }

  # get filename for display on output 
  if (.Platform$file.sep=="/"){
    m=regexec("[^/]*$",fileName)            #  ./(.$)    pc=[^\\]*$   mac=[^/]*$
  } else {
    m=regexec("[^\\]*$",fileName)
  }
  fileName1<-regmatches(fileName,m)
  
  # send output to file  -- build name of file with path to data f
  BaseTime<-Sys.time()
  thisTime <- format(BaseTime, "--%d%b%y--%H-%M-%OS")
  fileName2<-paste(fileName,window,thisTime,"Cos.txt",sep="")
  fileName3<-paste(fileName,window,thisTime,"Cos.rtf",sep="")
  if (Output$Txt){
    sink(fileName2, append=FALSE, split=TRUE)
  }
  if (!Console){   # if console is set to off, check the type of output for graphics
      if (Graphics == "pdf"){
        fileName4<-paste(fileName,window,thisTime,"Cos.pdf",sep="")
        pdf(file=fileName4,width=8, height=10)
      } else {if (Graphics == "jpg"){
                fileName4<-paste(fileName,window,thisTime,"Cos.jpg",sep="")
                jpeg(filename=fileName4,width=8, height=10)
                } else {if (Graphics == "png"){
                          fileName4<-paste(fileName,window,thisTime,"Cos.png",sep="")
                          png(filename=fileName4,width=8, height=10)
                        } else {if (Graphics == "tif"){
                          fileName4<-paste(fileName,window,thisTime,"Cos.tif",sep="")
                          tiff(filename=fileName4,width=8, height=10)
                        }
                }
                }
      }
  }
  
  if (timeFormat!="%Y%m%d%H%M"){

      if (header==TRUE){
        myHeader <- read.table(fileName, header=TRUE, nrows=1, sep="\t",skip=Skip)
        #get the header for names  -- all strings matching header will be removed
        headerChar1 <- substr(names(myHeader)[1],1,1)    # gets the 1st character of the header to be used as a comment indicator
        MyDataReada <- read.table(fileName, header=header, sep="\t",skip=Skip,stringsAsFactors=FALSE,comment.char=headerChar1,col.names = names(myHeader))
        
      } else {headerChar1 <- ""             #  use NA as a comment indicator
              MyDataReada <- read.table(fileName, header=header, sep="\t",skip=Skip,stringsAsFactors=FALSE,comment.char=headerChar1)          
      }  
      MyDataRead<-subset(MyDataReada, !is.na(MyDataReada[,Y]))       # omit data row if Y column is NA

      #MyDataRead = na.omit(head(MyDataRead1,n=-2))   #  will drop the very last row in case it had no data
    
      if (timeFormat=="numeric"){
        MyDataRead[,TimeCol]<-format(BaseTime + as.numeric(MyDataRead[,TimeCol])*3600,"%Y%m%d%H%M")   # convert to hours, and add to today's date.
        #RefDateTime<-MyDataRead[1,TimeCol]
      } else {   #convert time from 2 columns
        if (length(TimeCol)==2){
          MyDataRead[,TimeCol[1]] <- as.POSIXct(strptime(x=paste(MyDataRead[,TimeCol[1]],MyDataRead[,TimeCol[2]]), format="%d/%m/%y %H:%M:%S", tz=tz))
          MyDataRead[,TimeCol[2]] <- format(MyDataRead[,TimeCol[1]],"%Y%m%d%H%M")
          names(MyDataRead)[TimeCol[2]] <- "Datetime"
          TimeCol<-TimeCol[2]
        } else {   #convert time from 1 columns
          if (length(TimeCol)==1){
            MyDataRead[,TimeCol[1]] <- as.POSIXct(strptime(x=MyDataRead[,TimeCol[1]], format=timeFormat, tz=tz))
            MyDataRead[,TimeCol[1]] <- format(MyDataRead[,TimeCol[1]],"%Y%m%d%H%M")
            names(MyDataRead)[TimeCol[1]] <- "Datetime"
          }}}
      
      # end timeFormat not "%Y%m%d%H%M"
    } else {  # timeFormat=="%Y%m%d%H%M"
 
      tableColTypes<-c("character","numeric","numeric","numeric")       # ,"numeric","character")   # only char, num sometimes
      if (TimeCol>=1 && TimeCol<9) {
        tableColTypes[TimeCol]<-"character"
      } else stop("ERROR:  Are you sure your Time column is > 9?")
    
      MyDataReada=read.table(fileName,header=header,sep=',',stringsAsFactors=F, colClasses=tableColTypes,blank.lines.skip=TRUE,skip=Skip,na.strings=c(" ","--"))
      for (y in Y){
        MyDataRead<-subset(MyDataReada, !is.na(MyDataReada[,y]))       # omit data row if Y column is NA
      }
      missingData<-length(MyDataReada)-length(MyDataRead)

      if (timeFormat=="numeric"){
        MyDataRead[,TimeCol]<-format(BaseTime + as.numeric(MyDataRead[,TimeCol])*3600,"%Y%m%d%H%M")
        #RefDateTime<-format(BaseTime + as.numeric(RefDateTime)*3600,"%Y%m%d%H%M")
      }
      } # end reading for format 2
      
  str(MyDataRead)
  AnalyzeLength<-(as.POSIXct(strptime(tail(MyDataRead[,TimeCol],1), "%Y%m%d%H%M",tz=tz))+3600) - as.POSIXct(strptime(MyDataRead[1,TimeCol], "%Y%m%d%H%M",tz=tz)) 
  MyData_HRs<-(AnalyzeLength)*24  # hours in length of actual data

  if (StartDate == 0){
    StartDate <- format(as.POSIXct(strptime(MyDataRead[1,TimeCol], "%Y%m%d%H%M",tz=tz)),"%Y%m%d%H%M")
  } else {if (StartDate == 12){
    StartDate <-paste(substr(MyDataRead[1,TimeCol],1,8),"0000",sep="")                #  Use StartTime as start of analysis date-time
  }} #as.POSIXct(strptime(MyData[1,TimeCol], "%Y%m%d",tz=tz))
  MyData_length<-length(MyDataRead[,1])

  StartIndex<-which(MyDataRead[,TimeCol]>=StartDate)   #  where is the desired startTime?  (if start point is later than first data point)
  if (EndDate == 0){
    EndDate <- format(as.POSIXct(strptime(tail(MyDataRead[,TimeCol],1), "%Y%m%d%H%M",tz=tz))+3600,"%Y%m%d%H%M") # should these be pasted or posix?  todo
    } else {if (EndDate == 12){
    EndDate <-paste(substr(as.numeric(MyDataRead[MyData_length,TimeCol])+as.numeric(10000),1,8),"0000",sep="")                #  Use StartTime as start of analysis date-time
  } }

  EndIndex<-which(MyDataRead[,TimeCol]<=EndDate)   #  where is the desired endTime? 
  cat("Check that your data file and End Date date are correct.  Data file begins:",MyDataRead[1,TimeCol],"; End Date is: ",EndDate,"\n")
  
  MyData<-MyDataRead[StartIndex[1]:tail(EndIndex,n=1),]     #  grab all the data from array after StartTime, before EndTime
  
  cat("There are ",MyData_HRs," actual hours of data in this file, ",MyDataRead[1,TimeCol]," to ",MyDataRead[tail(EndIndex,n=1),TimeCol],", and ",MyData_length,"data points.\n  Analysis will proceed from ",StartDate," to ",EndDate,"\n")

  print(MyDataRead[1,])
  print(tail(MyDataRead,1))
  # convert time
  MyData$time=as.POSIXct(strptime(MyData[,TimeCol], "%Y%m%d%H%M", tz=tz))     #   tz="GMT"
  # convert time to a number (time in seconds, converted into hours)
  # should probably be done with a hard coded time zone, so CDT and DST don't cause problems

  if (RefDateTime == '0'){
    RefTime <-as.POSIXct(strptime(MyData[1,TimeCol], "%Y%m%d%H%M",tz=tz))                #  Use StartTime as reference date-time
    RefTimeString <- MyData[1,TimeCol]
  } else if (RefDateTime==12){
    RefTime<-as.POSIXct(strptime(MyData[1,TimeCol], "%Y%m%d",tz=tz))     # midnight of first day of data
    RefTimeString <- paste(substr(MyData[1,TimeCol],1,8),"0000",sep="")
     } else {RefTime<-as.POSIXct(strptime(RefDateTime, "%Y%m%d%H%M",tz=tz))
             RefTimeString <- RefDateTime}

  StartTime<-(as.numeric(as.POSIXct(strptime(StartDate, "%Y%m%d%H%M",tz=tz)))-as.numeric(RefTime))/3600      # convert to hours
  EndTime<-(as.numeric(as.POSIXct(strptime(EndDate, "%Y%m%d%H%M",tz=tz)))-as.numeric(RefTime))/3600   # convert to hours
  
  MyData_hours<-as.numeric(EndTime-StartTime)           #   get number of hours to analyze
  
  MyData$time.n=as.numeric(MyData$time) - as.numeric(RefTime)
  MyData$time.hour<-MyData$time.n/3600
  MyData_length<-length(MyData[,1])
  #MyData_hours<-(MyData$time.hour[MyData_length]-MyData$time.hour[1])  # hours in length of data if start date is after 1st data point

  # calculate number of hours in the full regression (Units*RegressionLen) -- it may not use all of the data
  # this is used for outer loop, full analysis
  #if (Units=='Day'){
  #  MyData_hours<-RegressionLen*24
  #} else if (Units=='Year'){
  #        MyData_hours<-24*365*RegressionLen
  #        } else if (Units=='Week'){
  #                MyData_hours<-24*7*RegressionLen    
  #              }
  cat("The program is using ",MyData_hours," hours for the analysis, starting at hour ",StartTime," relative to RefTime: ",RefTimeString,"\n")
  # calculate number of hours in the Span and Interval (Units*Span, Units*Interval)
  # this is used for inner loop, progressive analysis of intervals over the full data set
  if (Span==0 || Increment==0){     #  || oneCycle[1]>0){     #  No progression -- analyze full dataset
    Span <-MyData_hours
    Increment<-MyData_hours
  } else if (Units=='Hour'){
        Span<-Span
        Increment<-Increment
      } else if (Units=='Day'){
            Span<-Span*24
            Increment<-Increment*24
          } else if (Units=='Year'){
                  Span<-Span*24*365
                  Increment<-Increment*24*365
                } else if (Units=='Week'){
                        Span<-24*7*Span      
                        Increment<-24*7*Increment
                      } else stop(paste("ERROR:  Units is improperly given: ",Units," Valid values:  Hour, Day, Week, Year"))

  if (FreqStart==0){       
    FreqStart <-Span
  } else {if (FreqStart>MyData_hours){        #  validation of FreqStart   (Span chnaged to MyData_hours)
            stop("ERROR:  The parameter Period$Start cannot be larger than Progressive$Span")
          }
          if (FreqStart<0){
            stop("ERROR:  The parameter Period$Start cannot be negative")
          }
      }  # end else FreqStart==0
  #FreqEnd<-FreqStart/(Period$End*FreqInc)
  if (FreqInc==0){
    FreqInc<-1
  }
  if (FreqEnd==0){
    FreqEnd<-4
    if (FreqEnd>FreqStart){
      FreqEnd<-FreqStart/4
      FreqInc<-.1
    }
  }
  cat(FreqStart," FreqStart ",FreqEnd," FreqEnd ",Period$Start,"Period$Start",Span,"Span",Increment,"Increment\n")

  #Variable<-paste(fileName1,":V",Ycol,sep="")
  opar = par()
  par(mar=c(4,4,1,1)) # set up margins
  
  #  #####    note on indexes and variables:  one set tracks through the ACTUAL times and data points:  #####
  #                EndIdx, StartIdx, thisIdxCnt, MyData_length, TimeIdx
  #  #####    another set tracks through the periods, in hours, being calculated:       ######
  #                MyData_hours; StartSpans, Span, Increment, Progression_end, StartTime, StartSpanTime, EndTime
  #  #####    These two must, of course, align. See:   TimeIdx<-which(StartSpanTime <= MyData$time.hour) 
  EndIdx<-0
  # seq(72-84+1,192-50.4,by=8.4)
  if (MyData_hours==Span){    #StartTime==MyData$time.hour[1] && 
    StartSpans<-seq(from=0,to=MyData_hours-1,by=Increment)        #StartSpans<-seq(1,Span, by=Increment)
  #} else {StartSpans<-seq(from=(StartTime),to=(StartTime+MyData_hours-Span),by=Increment)  
  } else {StartSpans<-seq(from=0,to=(MyData_hours-Span+Increment),by=Increment)     #  added + Increment because 3varsProg was doing 1 too few spans
          }   #StartSpans<-seq(1,Span, by=Increment)
  print(StartSpans)
  Progression_end<-length(StartSpans)
  Page<-0

  if (oneCycle[1]==0){                    #  if oneCycle != 0, FreqEnd and FreqInc are not used.
    if (FreqEnd <= 0 || FreqInc <= 0){
      stop(paste("ERROR:  Period$End (",FreqEnd,") and Period$Increment (",FreqInc,") cannot be 0 when Period$Set is 0"))
    }
    if (FreqStart<FreqEnd){
      stop(paste("ERROR:  The parameter Period$Start (",FreqStart,") cannot be smaller than to Period$End (",FreqEnd,") when Period$Set is 0"))
    }
    if (FreqInc > FreqStart){
      stop(paste("ERROR:  Period$Increment (",FreqInc,") cannot be larger then Period$Start (",FreqStart,") (or Progressive$Span) when Period$Set is 0"))
    }
    RowCnt<-(FreqStart/FreqEnd)/FreqInc       #   (FreqStart-FreqEnd)/FreqInc  #  (Span/2)/FreqInc
  } else {if (Components>1){
              RowCnt<-1}
          else {RowCnt <- length(oneCycle)}   #  if oneCycle != 0, reset FreqEnd and FreqInc to default=1
          FreqStart <-Span
          FreqInc <-1
          print("resetting freq start and inc")
  }
  
  M <- matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  PHI <- array(data=NA,c(RowCnt, Progression_end,Components))
  PHIr <- array(data=NA,c(RowCnt, Progression_end,Components))
  A <- array(data=NA,c(RowCnt, Progression_end,Components))
  PR <- array(data=NA,c(RowCnt, Progression_end,Components))
  P <- array(data=NA,c(RowCnt, Progression_end,Components))
  testPg <- array(data=NA,c(RowCnt, Progression_end,Components))
  testPb <- array(data=NA,c(RowCnt, Progression_end,Components))
  F <- array(data=NA,c(RowCnt, Progression_end,Components))
  mesor_se <- matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  phi_se <- array(data=NA,c(RowCnt, Progression_end,Components))
  amp_se <- array(data=NA,c(RowCnt, Progression_end,Components))
  Cycle <- array(data=NA,c(RowCnt, Progression_end,Components))
  hours <-matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  nPts<- matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  time<- matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  MSS<- vector(length = Components)
  if (Components>1){
    multi_P<-matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
    multi_PR<-matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  }
  newData<- matrix(data=NA,nrow=RowCnt, ncol=Progression_end)    # holds filtered or unfiltered data, myData for each Span

  for (y in 1:Ys_end){
    Ycol<-Y[y]
    #print(Ycol)
    # Use the periodogram
    if (window!="noFilt"){
      CatWindowList<-CatWindow(window=window,myData=MyData[,Ycol],Time=MyData$time.hour,dataN=MyData_hours,Start=MyData$time.hour[1]+StartSpans[1]-1)    # plots the full set of data w/filter
    }
    
    layout(matrix(c(1,2), 2, 1, byrow = TRUE), 
          widths=c(1), heights=c(1,2))          #   c(figure#1, figure#2), 2 row, 1 col, ...
    par(mar=c(4,4,2,1), oma=c(2,0,1,0))         # set up margins
    if (dt!=0 && window!="noFilt"){
      CatPeriodogram<- CatPeri(CatWindowList$data,binsPerHr=1/dt, export=T, fileoutP=paste(fileName,thisTime,"V",Ycol,"CosPgram.txt"), Hx=.15, cex=.9, Debug=FALSE)    # pass samples/day when units=hrs
      View(CatWindowList$data)
    } else {plot.new()}

    n<-length(MyData[,Ycol])
    if (is.numeric(MyData[n, 1])){
      startDateP<-as.POSIXct(strptime(x=MyData[1,1], format="%Y%m%d%H%M", tz=tz))
      endDateP<-as.POSIXct(strptime(x=MyData[n,1], format="%Y%m%d%H%M", tz=tz))
    } else {startDateP<-MyData[1,1]
            endDateP<-MyData[n,1]}
    title(paste('Start time:',startDateP,';    End time:',endDateP),cex=.7)
    #-------------------
#     if (Plex==TRUE){
#       CatPlexigram<- CatPlex(CatWindowList$data, foldLen=foldLen, Debug=FALSE) 
#     }

      cat("\n\tPR\t\t   F\t\t\tP \tSS[j]\t i\t cycle \t  Mesor \t  s.e. \t \t  Amp \t\t s.e. \t \tPhi\t\t s.e.\n")
      #cat("-------------------------------------------------------\n")   
      for (j in 1:Progression_end){   #  make a vector with incremental values.  use var(i) for necessary indexes
      # from:to
        if (j %in% c(1,10)){
          if (j!=1){
            par(mfrow=c(1,1),mar=c(4,4,4,1))
            jht<-2.5
            cexVar<-.5
            htVar<-.3
          } else {jht<-0
              cexVar<-.7
              htVar<-.8}
          Page<-Page+1
          if (oneCycle[1]>0){
            main<-paste("Each Period: Var",y)
            } else {main<-paste("COSINOR Maximum Period: Var",y)}    
          
          plot(0:10, 0:10, type="n",xlab="",ylab="",axes=FALSE, main=main,cex=1)
          text(c(0,.7,1.5,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3),c(9.5,9.5,9.5,9.5,9.5,9.5,9.5),labels=c("Span","Period","F","P","PR","Mesor", "s.e.","Amp","s.e.","Phi","s.e."),adj = c(0,0))   #  adj = c(0,0) left justifies
          abline(h=9)
          #page footer
          mtext(paste('Function:',functionName,"  ",fileName1,"; TimeCol=",TimeCol," Ycol=",Ycol, " RefDateTime=",RefDateTime),side=1,line = 1, cex =.5, adj=.9)
          mtext(paste("P",Page,"  ",window,"  ",thisTime," Units=",Units, " Span=",Span, " Increment=",Increment, " header=",header," Set=",Period["Set"]),side=1,line = 2, cex =.5, adj=.9)
        }
          maxAmp = 1                          
          StartSpanTime<-StartTime+StartSpans[j]    # Feb 2014 added back in; 11/4  StartTime+StartSpans[j]-1 # 10/18 added StartTime+ DH;  9/30  MyData$time.hour[1]+StartSpans[j]-1
          EndSpanTime<-StartSpanTime +Span      # 9/30  -1***adj to match gc  Add span length to the last span StartSpans[j]+Span to get the ending of the span
          TimeIdx<-which(StartSpanTime <= MyData$time.hour)   #   get indices past the start of this span
          StartIdx<-TimeIdx[1]                # index of starting time is FIRST index in array of indices
             #   when times from 2 to 23 and EndSpanTime=22, which(EndSpanTime >= MyData$time.hour) doesn't include time of 23, but should  (BlissT17-1.dat.txt)
          #EndSpanTime<-StartTime+EndSpan        # *Feb 2014 added back in, then removed; 10/18 restored to: 9/30 StartTime + EndSpan-1  taken out  # EndSpan is # hours to end of this span;  add to starting time
          if (EndSpanTime>MyData_hours+StartTime){   # Feb 2014 had to add + StartTime to these 2 lines
            EndSpanTime<-MyData_hours+StartTime
          }
          #if (EndSpan>EndTime){
          #  print("**************what to do about this???****************")
          #  EndSpan<-MyData_hours
          #}
          TimeIdx<-which(EndSpanTime >= MyData$time.hour)   #  should be gte;  get indices UP TO the end of this span
          EndIdx<-which.max(TimeIdx)               # index of ending time is LAST index in array if indices
          
          thisIdxCnt<-EndIdx-StartIdx+1   # data points in this span  -- used for S11, and SigmaHat
          Loops<-RowCnt          
          if (Components==1 && oneCycle[1] > 0){              #  a specified period in oneCycle overrides spectrum calculation  (and FreqStart/End?)
            Cycles<-oneCycle
            Loops<-length(Cycles)        
          } else if (Components>1){
            Cycles<-oneCycle
            Loops<-1
          }
          #cat("StSpan ",StartSpans[j]," Span ",Span," EndSpan ",EndSpan," StrtIdx ",StartIdx, " EndIdx ",EndIdx," thisIdxCnt ",thisIdxCnt," j ",j,"\n")
        if (StartIdx<2 | EndIdx>1370){
          #browser()
        }
        if (window != "noFilt"){          # calculates a window for the Span
          CatWindowList<-CatWindow(window=window,myData=MyData[StartIdx:EndIdx,Ycol],Time=MyData$time.hour[StartIdx:EndIdx],dataN=Span,Start=StartSpanTime)
          newData<-CatWindowList$data
        }
        else {newData<-MyData[StartIdx:EndIdx,Ycol]}

          for (i in 1:Loops){
            # fit sinusoidal model
            if (oneCycle[1]==0){                  #  if a period has been specified, use present cycle
              #cycle<-Span/(i-(StartSpans[j]-1))       #  divisor should start at one each time this loop restarts
              cycle<-FreqStart/(1+((i-1)*FreqInc))          ####  Span/(1+((i-1)*FreqInc))  from Span/i
              } else cycle<-Cycles[i]
            
            Cycle[i,j,1]<-cycle                    # store for later 
            hours[i,j] <- paste(format(StartSpanTime,nsmall=1)," - ",format(EndSpanTime,nsmall=1),sep="")  # took :59 off because can be decimal hrs;  this shows ALL used
            nPts[i,j] <- paste(StartIdx,"-",EndIdx,"\n(",EndIdx-StartIdx+1,")")
            time[i,j] <- paste(MyData[StartIdx,TimeCol],MyData[EndIdx,TimeCol])

            #array of parameters
            #MN,             Bcos(2pi/tau)t,               Gsin(2pi/tau)t
            #Mcos(2pi/tau)t, Bcos^2(2pi/tau)t,             Gcos(2pi/tau)t*sin(2pi/tau)t
            #Msin(2pi/tau)t, Bcos(2pi/tau)t*sin(2pi/tau)t, Gsin^2(2pi/tau)t

            CosMatrix<-matrix(data=NA,nrow=CosMatrixDim, ncol=CosMatrixDim)
            testXmean<-matrix(data=NA,nrow=CosMatrixDim, ncol=Components)
            testX<-matrix(data=NA,nrow=CosMatrixDim, ncol=Components)
            testXprod<-vector(length=CosMatrixDim)
            YMatrix<-matrix(data=NA,nrow=CosMatrixDim, ncol=1)
            CosCoMatrix1<-matrix(data=NA,nrow=CosMatrixDim, ncol=thisIdxCnt)
            CosCoMatrix1[1,]<-rep(x=1,times=thisIdxCnt)         # need to be 0s for CosMatrix<-Sum...

            for (m in 1:Components){
              if (Components>1){    #  if components >1, override value of cycle set above.
                cycle<-Cycles[m]
                Cycle[i,j,m]<-cycle
              }
              CosCoMatrix1[m*2,]<-cos(MyData$time.hour[StartIdx:EndIdx]*(2*pi)/cycle)     # x1 for  cycle m
              CosCoMatrix1[m*2+1,]<-sin(MyData$time.hour[StartIdx:EndIdx]*(2*pi)/cycle)   # z1 for  cycle m
              YMatrix[m*2] <- sum(newData%*%(cos(MyData$time.hour[StartIdx:EndIdx]*(2*pi)/cycle)))     # Y*x1
              YMatrix[m*2+1] <- sum(newData%*%(sin(MyData$time.hour[StartIdx:EndIdx]*(2*pi)/cycle)))   # Y*z1 
            }
            #cat("cycle",cycle,"Cycles",Cycles)

            # vector multiplication to build matrix for cosine
            for (m in 1:CosMatrixDim){      #  columns
              for (o in 1:CosMatrixDim){    #  rows
                  CosMatrix[o,m]<-sum(CosCoMatrix1[m,] * CosCoMatrix1[o,])
              }
            }
            CosMatrix[1,1]<-thisIdxCnt
            YMatrix[1] <- sum(newData)
            View(CosMatrix)
            View(YMatrix)
                      
            mdat<-CosMatrix

            mdat2<-YMatrix

            #invert matrix to get the solution
            if (det(mdat)<.0000001){
              print("The matrix cannot be inverted")
            }  else{mdatInv <- solve(mdat)
            
            #multiply the inverted matrix by the dependent variable matrix to get vector M, B, G, B2, G2, ...   (x=inv(S)b)
            coefs <- mdatInv  %*%  mdat2
            #browser()
            if (Debug==TRUE || i==Span){
                #print(TimeIdx)
              #print(crsprd)
              #print(s)
              #print(model$coefficients)
              #print(Amp)
              #cat("data22 ",MyData$time.hour[StartIdx:EndIdx]*(2*pi)/cycle," cycle ",cycle,"\n")
              #cat("cos22 ",cos(MyData$time.hour[StartIdx:EndIdx]*(2*pi)/cycle)," cycle ",cycle,"\n")
              #cat("StartIdx ",StartIdx,"  EndIdx ",EndIdx," thisIdxCnt ",thisIdxCnt,"\n")
                print(mdat)
                print(mdat2)
                #print(mdatInvChol)
                print(mdatInv)
                cat("matrix coefs ",coefs,"\n")
                #browser()         #########   Great Debug location
            }
            #calculate MESOR only once, regardless of # of components
            M[i,j]<-coefs[1]
            #calculate the Mean of Ys
            MeanR<-mean(newData)
            multi_Model_Y<-Model_Y <- M[i,j]          # Model_Y = Mesor +  bX + gZ 
                    
            for (m in 1:Components){
              beta<-m*2
              gamma<-m*2+1
              if (Components>1){    #  if components >1, override value of cycle set above.
                cycle<-Cycles[m]
                Cycle[i,j,m]<-cycle
              }
                #sometimes coefs will be undefined.  skip calculations and move to next cycle
                if (!is.na(coefs[beta]) && !is.na(coefs[gamma])){
                    #Calculate Amplitude using B and G   (coefs[beta]=Beta)
                    A[i,j,m]<-(coefs[beta]^2+coefs[gamma]^2)^0.5
                    
                    #preserve the index for the cycle with the maximum amplitude           
                    if (A[i,j,m]>A[maxAmp,j,m]) {
                      maxAmp<-i
                    }            
                      # Y(t) = M + bX + gZ    where X=cos(t2pi/cycle) and Z = sin(t2pi/cycle)
                      Model_Y = M[i,j] + (coefs[beta]*cos(2*pi*MyData$time.hour[StartIdx:EndIdx]/cycle)) + (coefs[gamma]*sin(2*pi*MyData$time.hour[StartIdx:EndIdx]/cycle))
                      if (Components>1){      # multiple components model 
                        multi_Model_Y = multi_Model_Y + (coefs[beta]*cos(2*pi*MyData$time.hour[StartIdx:EndIdx]/cycle)) + (coefs[gamma]*sin(2*pi*MyData$time.hour[StartIdx:EndIdx]/cycle))
                      }

                     MYhat_Ymean<-Model_Y-MeanR     #   calculates MSS for single component model
                     MSS[m]<-sum(MYhat_Ymean^2)        #   calculates MSS for single component model
                    
                    #calculate phi:  atan(-G/B) + Kpi
                    ph<-atan(-coefs[gamma]/coefs[beta])
                    if (coefs[beta]>=0) {phi2<-ph}
                    if (coefs[beta]<0&coefs[gamma]>=0) {phi2<-ph+pi}
                    if (coefs[beta]<0&coefs[gamma]<0) {phi2<-ph-pi}
                    # put in 0 to 2pi range
                    PHIr[i,j,m]<-phi2
                    if (phi2<0) { PHIr[i,j,m]<-phi2+(2*pi)} 
                    if (phi2>(2*pi)) { PHIr[i,j,m]<-phi2-(2*pi)}
                    # convert to degrees betwen 0 and 360
                    PHI[i,j,m] <- (PHIr[i,j,m]/(2*pi))*360
                    if (PHI[i,j,m]>0) {PHI[i,j,m]<-PHI[i,j,m]-360}
                    
                }   #  end coefs cannot be calculated
            }   # end coef calculation for each component
            }   # end matrix cannot be inverted
            
            # sum(Y-meanY)^2 = sum(Y - Yhat)^2 + sum(Yhat - meanY)^2
            if (Components>1){      # multiple components model 
              Model_Y <- multi_Model_Y       #  use the full model for calculations from here on
              # calculate MSS for the full model
              MYhat_Ymean<-Model_Y-MeanR
              multi_MSS<-sum(MYhat_Ymean^2)
            }
            MY_Yhat <- newData-Model_Y
            RSS<-sum(MY_Yhat^2)
            df1<-Components*2
            df2<-thisIdxCnt-CosMatrixDim                      # N-2*Components+1

            if (Components>1){      # multiple components model 
              multi_F<-(multi_MSS/df1)/(RSS/(df2))       #  df1 = 2, since MSS is for one component;  df2 is for full model
              multi_P[i,j]<-1-pf(multi_F,df1=df1, df2=df2)              # Fisher–Snedecor (F ) (χ2 ) 
              multi_PR[i,j]<-(multi_MSS/(RSS+multi_MSS))*100
              
              #calculate magnitude:  if all components are multiples of largest, divide one period into 360 (outside loop)
              #       otherwise, divide full length of data hours into 360;  -->dT     Starttime+dT-->T
              if (LCM<=Span){
                dTime<-LCM/360        # length of time units for each degree of one period
              } else {
                  dTime<-Span/360
              }
              magTime<-vector(length = 360)
              magTime<-seq(from=MyData$time.hour[StartIdx],by=dTime,to=(MyData$time.hour[StartIdx]+(360*dTime)))
              multi_Model_Y_mag<- M[i,j]
            } 
           
            SigmaHat<-(RSS/(df2))^.5
            #cat(thisIdxCnt," sHat ",SigmaHat,"\t",PR[i,j],"\t",F[i,j],"\t ",P[i,j],"\t ")

            mesor_se[i,j] <-SigmaHat * mdatInv[1,1]^.5

            #   for testing alternate formula for individual P
            for (m in 1:Components){
              F[i,j,m]<-(MSS[m]/2)/(RSS/(df2))       #  df1 = 2, since MSS is for one component;  df2 is for full model
              P[i,j,m]<-1-pf(F[i,j,m],df1=2, df2=df2)              # Fisher–Snedecor (F ) (χ2 )

              # testing alternate formula for individual P
              tInd1<-2*m
              tInd2<-2*m+1
              beta<-2*m
              gamma<-2*m+1
              #   excludes diagonals of inverse matrix, which may be important
              test<-(mdatInv[tInd2,tInd2]*coefs[beta]^2)+2*mdatInv[tInd1,tInd2]*coefs[beta]*coefs[gamma]+mdatInv[tInd1,tInd1]*coefs[gamma]^2
              test2<-test/(mdatInv[tInd1,tInd1]*mdatInv[tInd2,tInd2]-mdatInv[tInd1,tInd2]^2)
              test3<-test2/(2*SigmaHat^2)
              cat("test3",m,"   ",test3,"\n")
              testPg[i,j,m]<-1-pf(test3,df1=2, df2=df2) 
              cat("testPg",m,"   ",testPg[i,j,m],"\n")
              #  alternative calculation for F    p 409 Bingham [33] 
              testXmean<-(CosMatrix[1,tInd1])/thisIdxCnt    #  X mean
              testZmean<-(CosMatrix[1,tInd2])/thisIdxCnt    #  Z mean
              testX2<-sum((CosCoMatrix1[tInd1,] * CosCoMatrix1[1,]-testXmean)^2)   #  X 
              testZ2<-sum((CosCoMatrix1[tInd2,] * CosCoMatrix1[1,]-testZmean)^2)   #  Z 
              testXZ<-sum((CosCoMatrix1[tInd1,] * CosCoMatrix1[1,]-testXmean)*(CosCoMatrix1[tInd2,] * CosCoMatrix1[1,]-testZmean))   #  XZ 
              testF<-(testX2 * coefs[beta]^2 + 2*testXZ * coefs[beta] * coefs[gamma] + testZ2 * coefs[gamma]^2)/(2*SigmaHat^2)

              cat("testF",m,"   ",testF,"\n")
              testPb[i,j,m]<-1-pf(testF,df1=2, df2=df2) 
              cat("testPb",m,"   ",testPb[i,j,m],"\n")
                browser()
              if (Components>1){      # multiple components model 
                PR[i,j,m]<-(MSS[m]/(RSS+multi_MSS))*100
                #Y(t) = M + bX + gZ + e(t)where  b = Acosf  and  g = -Asinfand  X = cos(2pt/t)  and Z = sin(2pt/t)
                multi_Model_Y_mag = multi_Model_Y_mag + coefs[beta]*cos(2*pi*magTime/Cycles[m]) + coefs[gamma]*sin(2*pi*magTime/Cycles[m])
                } else {
                PR[i,j,m]<-(MSS[m]/(RSS+MSS[m]))*100
              }
              
              #calculate standard error for mesor, amplitude, phi -- convert se for phi to degrees
              amp_se[i,j,m] <-SigmaHat * (mdatInv[beta,beta] * cos(PHI[i,j,m])^2 - (2 * mdatInv[beta,gamma] * cos(PHI[i,j,m]) * sin(PHI[i,j,m])) + mdatInv[gamma,gamma] * sin(PHI[i,j,m])^2)^.5
              phi_se[i,j,m] <- (SigmaHat * (mdatInv[beta,beta] * sin(PHI[i,j,m])^2 + (2 * mdatInv[beta,gamma] * cos(PHI[i,j,m]) * sin(PHI[i,j,m])) + mdatInv[gamma,gamma] * cos(PHI[i,j,m])^2)^.5)/A[i,j,m]
              phi_se[i,j,m]<-phi_se[i,j,m]*180/pi       # Convert to degrees
              if (Debug==TRUE){
                cat(" seM ",mesor_se[i,j]," seA ",amp_se[i,j,m]," sePHI ",phi_se[i,j,m]," \n")
              }
              cat(thisIdxCnt,"\t","sigma ",SigmaHat,"\t",PR[i,j,m],"\t",F[i,j,m],"\t ",P[i,j,m],"\t \n")
              cat(StartSpans[j],"\t",i,"\t",format(Cycle[i,j,m],width=3), "\t ",M[i,j],"\t", format(mesor_se[i,j],digits=8),"\t",A[i,j,m],"\t",format(amp_se[i,j,m],digits=8),"\t",format(PHI[i,j,m],digits=8),"\t",format(phi_se[i,j,m],digits=8),"\n")
              if (oneCycle[1]>0){   # print this cycle result if  oneCycle parameter set is>0
                ht<-10-(i+(j*htVar))+jht-(m-1)
                print(ht)
                #  prints period and estimates for each period in a range
                text(c(-.25,.8,1.3,2.1,3.2,4.2,5.1,6.2,7.2,8.2,9.2),c(ht,ht,ht,ht,ht,ht,ht,ht,ht,ht,ht),labels=c(paste(StartSpans[j]," - ",format(StartSpans[j]+Span-1,nsmall=3)),format(Cycle[i,j,m],nsmall=2),format(F[i,j,m],digits=6),format(P[i,j,m],digits=6),format(PR[i,j,m],digits=6),format(M[i,j],digits=8), format(mesor_se[i,j],digits=6),format(A[i,j,m],digits=6),format(amp_se[i,j,m],digits=6),format(PHI[i,j,m],digits=6),format(phi_se[i,j,m],digits=6)),cex=cexVar,adj = c(0,0))
              }
            }  # end  Components loop

          }    #  end i=Loops for loop
          if (oneCycle[1]==0){   # print cycle results here at end if  oneCycle parameter set 0
            ht<-9-(j*htVar)+jht        #8    5, 10, 11, 20, 15, 25
            # prints only the max for each spectrum of frequencies
            text(c(-.25,.8,1.3,2.2,3.1,4.2,5.3,6.3,7.2,8.2,9.2),c(ht,ht,ht,ht,ht,ht,ht,ht,ht,ht,ht),labels=c(paste(StartSpans[j]," - ",format(StartSpans[j]+Span-1,nsmall=3)),format(Cycle[maxAmp,j,m],nsmall=2),format(F[maxAmp,j,Components],digits=6),format(P[i,j,Components],digits=6),format(PR[maxAmp,j,Components],digits=6),format(M[maxAmp,j],digits=6), format(mesor_se[maxAmp,j],digits=6),format(A[maxAmp,j,Components],digits=6),format(amp_se[maxAmp,j,Components],digits=6),format(PHI[maxAmp,j,Components],digits=6),format(phi_se[maxAmp,j,Components],digits=6)),cex=cexVar,adj = c(0,0))
            
          }    #  end oneCycle
      }    #   END for (j in 1:Progression_end){  

      ht<-9-(j*htVar)+jht 
      if (Components>1){   
        #text(c(5,7),c(ht,ht),labels=c(multi_F,multi_P,multi_PR))
        cat("F  ",multi_F,"P  ",multi_P[i,j],"PR  ",multi_PR[i,j])
      }
    if (nrow(A)>1 & ncol(A)>1 & Output$HeatMap){
      par(mfrow=c(1,1),mar=c(4,4,3,4))
      #heatmap(MyData$time.hour, A, Rowv = NA, Colv = NA, main = "heatmap")
      
      x <- 1/Cycle[1:(i),1,m]                 ###F    1/Cycle[1:(i),1] 
      y <- StartSpans+StartTime+(Span/2)      #  10*(1:ncol(A))
      #browser()
      if (Period$Set[1]==0){
        HeatYlab<-paste('Frequency: cycles/hour (',FreqStart,"to",FreqEnd,Units,"in",FreqInc,Units,"Intervals)")
      }else {HeatYlab<-paste('Frequency: cycles/hour (At',Period["Set"],"periods)")
      }
      startDateP<-as.POSIXct(strptime(x=StartDate, format="%Y%m%d%H%M", tz=tz))
      endDateP<-as.POSIXct(strptime(x=EndDate, format="%Y%m%d%H%M", tz=tz))
      lvlCnt<-10
      colCount<-100    #**  19 for hanning filter;  9 w/o filter
      Nscale<-((0:colCount)/colCount)
      Nbw<-gray(Nscale)
      HeatMain<-paste("Frequencies across",startDateP,"-",endDateP)
      HeatXlab<-paste(Span,Units,'Intervals, in',Increment,Units,"Increments")
      #**HeatLegend<-seq(from=min(A),to=max(A),length.out=(colCount+1))
      
      #HeatLegend<-c(0,.16/19,.32/19,.48/19,.64/19,.80/19,.96/19,1.12/19,1.28/19,1.44/19,3/19,5/19,6/19,8/19,10/19,12/19,14/19,16/19,18/19,1)*max(A)
      HeatLegend<-Nscale*max(A)      # use the number of levels needed, multiply by max, to get character labels for legend
      HeatLegend2<-c(HeatLegend,max(A)+max(A)/colCount)    # this is only needed for image breaks=HeatLegend2, where breaks needs to have one more element than legend
      HeatIndex<-rep(1:(colCount/lvlCnt),lvlCnt)
      HeatLegend3<-(HeatLegend2[HeatIndex==1])
      NbwFill<-(Nbw[HeatIndex==1])
      #HeatLegend<-format(HeatLegend2,digits=4)
      if (Colors=="BW"){
        ###image(y, x, t(A[1:i,,]), axes = FALSE, col=rev(Nbw),main=HeatMain,ylab=HeatYlab,xlab=HeatXlab,breaks=HeatLegend2) 
        image(y, x, t(A[1:i,,]), axes = FALSE, col=rev(Nbw),main=HeatMain,ylab=HeatYlab,xlab=HeatXlab) #format(max(A)+max(A)*.2,digits=4)))      ###F    fill=rev(Nbw)
        contour(y, x, t(A[1:i,,]), axes = FALSE, nlevels=lvlCnt, labels=HeatLegend3, add=TRUE, drawlabels=FALSE)     #format(max(A)+max(A)*.2,digits=4)))      ###F    fill=rev(Nbw)
        #col controls the colors used in graph;  breaks controls the range of values for each color
        par(xpd=TRUE)  # Do not clip to the drawing area
        lambda <- .025
        legend(par("usr")[2],(par("usr")[4]-par("usr")[3])*.5,xjust = 0, yjust = 0,lwd=3, lty=1, title="Amplitude", legend=format(HeatLegend3,digits=4),fill=rev(NbwFill),text.width = strwidth("1.00"),cex=.5)                
        #  legend controls the numbers and colors shown on the legend.  No relationship to actual unless same is used.
        
      }else {if (Colors=="Heat"){
        image(y, x, t(A[1:i,,]), axes = FALSE, col=rev(heat.colors(colCount)),main=HeatMain,ylab=HeatYlab,xlab=HeatXlab)      ###F 
        contour(y, x, t(A[1:i,,]), axes = FALSE, nlevels=lvlCnt, labels=HeatLegend3, add=TRUE, drawlabels=FALSE)     #format(max(A)+max(A)*.2,digits=4)))      ###F    fill=rev(Nbw)
        par(xpd=TRUE)  # Do not clip to the drawing area
        lambda <- .025
        legend(par("usr")[2],(par("usr")[4]-par("usr")[3])*.5,xjust = 0, yjust = 0,lwd=3, lty=1, title="Amplitude", legend=format(HeatLegend3,digits=4),fill=rev(heat.colors(lvlCnt+1)),text.width = strwidth("1.00"),cex=.5)                
      } else {image(y, x, t(A[1:i,,]), axes = FALSE, col=rev(terrain.colors(colCount)),main=HeatMain,ylab=HeatYlab,xlab=HeatXlab)      ###F 
              contour(y, x, t(A[1:i,,]), axes = FALSE, nlevels=lvlCnt, labels=HeatLegend3, add=TRUE, drawlabels=FALSE)     #format(max(A)+max(A)*.2,digits=4)))      ###F    fill=rev(Nbw)
              par(xpd=TRUE)  # Do not clip to the drawing area
              lambda <- .025
              legend(par("usr")[2],(par("usr")[4]-par("usr")[3])*.5,xjust = 0, yjust = 0,lwd=3, lty=1, title="Amplitude", legend=format(HeatLegend3,digits=4),fill=rev(terrain.colors(lvlCnt+1)),text.width = strwidth("1.00"),cex=.5)                
      }   
      }#   Colors
      ##op <- par(mar=c(0,0,3,0)+.1)
      #persp(x, y, z, 
      #      theta = 45, phi = 30, 
      #      shade = .5, 
      #      col = rainbow(N), 
      #      border = "green",
      #      main = "perspective plot, theta=45, phi=30")
      #par(op)
      
      #contour(t(A[1:i,,]), add = TRUE, drawlabels = FALSE)
      mtext(paste("0 =",RefTime),side=1,line = 3, cex =.7, adj=1)
      #x axis corresponds to row number and the y axis to column number, with column 1 at the bottom, 
      #i.e. a 90 degree counter-clockwise rotation of the conventional printed layout of a matrix.
      axis(1, at = y)                        #  axis(1, at = y+(Span/2)) didn't work!!
      axis(2, at = x)  # 1/Cycle[1:(i),1]
      box()
      #axis(1, 1:7, LETTERS[1:7], col.axis = "blue")
      #pretty(x,n=x/100,min.n=7)
      #X1<-axTicks(1)
      #Y1<-axTicks(2)
      #rug(X1,col= adjustcolor("red", 0.5), lwd = 4)
    }
    
    if (i>1 && Output$LineGraphs==TRUE){
      #par(mfrow=c(6,1),mar=c(2,4,1,1))
      
      layout(matrix(c(1,2,3,4,5,6), 6, 1, byrow = TRUE), 
             widths=c(1), heights=c(1,1,1,1,1,1))          #   c(figure#1, figure#2), 2 row, 1 col, ...
      par(mar=c(2,4,1,1), oma=c(2,0,1,0))         # set up margins
      
      
      if (FreqInc !=1){
        Ylable_inc<- paste("in increments of",FreqInc,"cycles")       
      } else {Ylable_inc <-" "}
      
      if (j>1){              #  J is time span(s) -- progression through span start to ends
        for (k in 1:Loops){   #  i/k is frequency calculations for each time span
          # all cycles repeated, so using only list of cycles from first loop, if more than one loop
          #plot(MyData$time.hour,MyData[,Ycol],type="l",ylab=paste("raw data"),main=paste("Metrics for Variable ",Ycol,"; periods of ",Cycle[k,1,m],Units, Ylable_inc))
          #window:  MyData[,Ycol] is no longer correct.  how to use newData? (MyData*window)
          #  I think this should use same time spans as in J!!!
          plot(MyData$time.hour[StartIdx:EndIdx],newData,type="l",ylab=paste("data*window"),main=paste("Metrics for Variable ",Ycol,"; periods of ",Cycle[k,1,m],Units, Ylable_inc))
          cat(k,j,"\n")
          plot(1:j,M[k,],type="l",ylab="MESOR",col=8)
          ampMax<-which.max(A[k,])
          seMax<-which.max(amp_se[k,])
          plot(1:j,A[k,],type="l",ylab="Amplitude",col=4,ylim=c(0,(A[k,ampMax]+amp_se[k,seMax])))
          points(1:j,(A[k,]+amp_se[k,]),type="p",pch=20,cex=.1,axes=F,col=3,ylim=c(-360,0))   
          points(1:j,(A[k,]-amp_se[k,]),type="p",pch=20,cex=.1,axes=F,col=3,ylim=c(-360,0))
          #lines(A[1,]+M[1,],type="l")
          plot(1:j,PHI[k,],type="l",ylab="Acrophase",col=6,ylim=c(-360,0))
          #par(new=T)
          #plot(1:j,PHI[k,]+phi_se[k,],type="p",pch=20,axes=F,col=3)   #   what is wrong here?
          #par(new=F)
          points(1:j,(PHI[k,]+phi_se[k,]),type="p",pch=20,cex=.2,axes=F,col=3)   
          points(1:j,(PHI[k,]-phi_se[k,]),type="p",pch=20,cex=.2,axes=F,col=3)   
          plot(1:j,PR[k,],type="l",ylab="Acrophase",col=6) 
          #browser()  #  what does phi + phi-se look like?
          
          plot(1:j,P[k,],type="l",ylab="P Value",col=9,ylim=c(0,.6))
          abline(h=.5,lty="dashed")
          
          #browser()     #  if progressive, plot(1:j, X[k,]);   otherwise, plot(1:i,X[k,])  except variable always w/time
          # http://cran.r-project.org/doc/contrib/Lemon-kickstart/kr_addat.html
        }       # end K to Loops
      }  else {                   # end if j>1
        CycleLen<-length(Cycle)
        cat("cycle",CycleLen)
        if (CycleLen>6){    #   if Cycle = 0 there is a long sequence, otherwise only a few
          Ylable<- paste("Metrics for Variable ",Ycol,": periods of ",Cycle[i],' to ',Cycle[1],Units, Ylable_inc)
        } else {
          Ylable<- paste("Metrics for Variable ",Ycol,": periods of ",list(Cycle[]),Units, Ylable_inc)
        }
        #plot(MyData$time.hour,MyData[,Ycol],type="l",ylab=paste("raw data"),main=Ylable)
        #window:  maybe this is OK, since only 1 timespan (j=1).  MyData[,Ycol] is no longer correct.  how to use newData? (MyData*window)
        plot(MyData$time.hour[StartIdx:EndIdx],newData,type="l",ylab=paste("data*window"),main=Ylable)
        plot(1/Cycle[CycleLen:1],M[CycleLen:1],type="l",ylab='MESOR',col=8)
        #ampMax<-which.max(A[k,])
        #seMax<-which.max(amp_se[k,])
        plot(1/Cycle[CycleLen:1],A[CycleLen:1],type="l",ylab="Amplitude",col=4)    #,ylim=c(0,(A[]+amp_se[])))
        points(1/Cycle[CycleLen:1],(A[CycleLen:1]+amp_se[CycleLen:1]),type="p",pch=20,cex=.6,axes=F,col=3)   #,ylim=c(-360,0))   
        points(1/Cycle[CycleLen:1],(A[CycleLen:1]-amp_se[CycleLen:1]),type="p",pch=20,cex=.6,axes=F,col=3)   #,ylim=c(-360,0))
        #lines(A[1,]+M[1,],type="l")
        plot(1/Cycle[CycleLen:1],PHI[CycleLen:1],type="l",ylab="Acrophase",col=6,ylim=c(-360,0))
        #par(new=T)
        #plot(Cycle[],PHI[k,]+phi_se[k,],type="p",pch=20,axes=F,col=3)   #   what is wrong here?
        #par(new=F)
        points(1/Cycle[CycleLen:1],(PHI[CycleLen:1]+phi_se[CycleLen:1]),type="p",pch=20,cex=.6,axes=F,col=3)   
        points(1/Cycle[CycleLen:1],(PHI[CycleLen:1]-phi_se[CycleLen:1]),type="p",pch=20,cex=.6,axes=F,col=3)   
        plot(1/Cycle[CycleLen:1],PR[CycleLen:1],type="l",ylab="Percent Rhythm",col=6) 
        #browser()  #  what does phi + phi-se look like?
        
        plot(1/Cycle[CycleLen:1],P[CycleLen:1],type="l",ylab="P Value",col=9,ylim=c(0,.6))
        abline(h=.5,lty="dashed")

        #}       # end K to Loops
      }  #  end else if j=1
      mtext("Green dots = S.E. standard error",side=1,line = 2, cex =.8, adj=.7,col=3)
    } # end if i>1 and LineGraphs == T
    #View(mat)
    
  
    if(Output$Doc){
      #library(rtf)
      if (y==1){
        output<-fileName3
        rtf<-RTF(output,width=11,height=8.5,font.size=10,omi=c(1,1,1,1))
        addParagraph(rtf,paste(fileName3,"\n  TimeCol=",TimeCol,",  Ycol=",Ycol, ",  header=",header,"\n --  Periods=",Period["Set"],", Units=",Units, ",  Span=",format(Span,nsmall=3), ",  Increment=",format(Increment,nsmall=3), "\nFreqStart=",format(FreqStart,nsmall=3), ",  FreqInc=",format(FreqInc,nsmall=3), ",  FreqEnd=",format(FreqEnd,nsmall=3), "\nRefDateTime=",RefDateTime, ", StartTime=",format(StartTime,nsmall=3),", EndTime=",format(EndTime,nsmall=3),"Rows with missing Y values:",missingData,"\n"))
        PRdigits<-2
        PMAdigits<-3
        PHIdigits<-1
        }

      printP<-format(P,digits=4,nsmall=PMAdigits)
      printP[P<.0005]<-c("<.001")
      nPtsX<-array(data=nPts,c(i,j,m))
      MX<-array(data=M,c(i,j,m))
      timeX<-array(data=time,c(i,j,m))
      hoursX<-array(data=hours,c(i,j,m))
      if (Components>1){
        for (n in 1:Progression_end){
          addParagraph(rtf,paste("\n column",y,":    MESOR=",format(M[,n],digits=2*PRdigits,nsmall=PMAdigits),"     MESOR s.e.=",format(mesor_se[,n],digits=2*PRdigits,nsmall=PMAdigits)))
          addParagraph(rtf,paste("\n           Time Pts=",time[,n],"     Hours from RefDateTime=",hours[,n],"     #Pts","Period in Hrs=",nPts[,n]))
          mat<-data.frame(matrix(c(format(Cycle[,n,],nsmall=PRdigits),format(PR[,n,],digits=2*PRdigits,nsmall=PRdigits),printP[,n,],testPg[,n,],testPb[,n,],format(A[,n,],digits=2*PRdigits,nsmall=PMAdigits),format(amp_se[,n,],digits=2*PRdigits,nsmall=PMAdigits),format(PHI[,n,],digits=2*PHIdigits,nsmall=PHIdigits),format(phi_se[,n,],digits=2*PHIdigits,nsmall=PHIdigits)),ncol=9))  #,dimnames=list(row_names,colName))
          names(mat)<-c("Period in Hrs","PR=MSS/ (RSS+MSS)","P","P(Ger)","P(Bin)","Amp","Amp SE","PHI","PHI SE") # format column names
          addTable(rtf,mat,font.size=12,row.names=FALSE,NA.string="--")
          printP_multi<-format(multi_P[i,n],digits=4,nsmall=PMAdigits)
          printP_multi[multi_P[i,n]<.0005]<-c("<.001")
          addParagraph(rtf,paste("\n  Overall:   PR=",format(multi_PR[i,n],digits=2*PRdigits,nsmall=PMAdigits),"      P=",printP_multi,"\n\n"))
        }
        } else {
        mat<-data.frame(matrix(c(timeX,hoursX,nPtsX,format(Cycle,nsmall=PRdigits),format(PR,digits=2*PRdigits,nsmall=PRdigits),printP,format(MX,digits=2*PRdigits,nsmall=PMAdigits),format(mesor_se,digits=2*PRdigits,nsmall=PMAdigits),format(A,digits=2*PRdigits,nsmall=PMAdigits),format(amp_se,digits=2*PRdigits,nsmall=PMAdigits),format(PHI,digits=2*PHIdigits,nsmall=PHIdigits),format(phi_se,digits=2*PHIdigits,nsmall=PHIdigits)),ncol=12))  #,dimnames=list(row_names,colName))
        names(mat)<-c("Time Pts","Hours from RefDateTime","#Pts","Period in Hrs","PR=MSS/ (RSS+MSS)","P","Mesor","Mesor SE","Amp","Amp SE","PHI","PHI SE") # format column names
        addTable(rtf,mat,font.size=12,row.names=FALSE,NA.string="--")
        
        }
      #mat<-data.frame(matrix(c(time,hours,nPts,Cycle,PR,P,M,mesor_se,A,amp_se,PHI,phi_se),ncol=12))  #,dimnames=list(row_names,colName))
      
      
    }    # end RTF
  }    #    END for (y in 1:Ys_end){
  done(rtf)                    # writes and closes the file
  if (Output$Txt){
    sink()
  }
  if (Output$TxtAmp){
    filename<-cat(fileName2,"amp")
    sink(fileName2, append=FALSE, split=TRUE)
    print(A[i])
    sink()
  }
  
  if (!Console){
    print(paste("Output has been saved to:",fileName4))
    dev.off()
  }

  #remove read only list components 
  opar$cin<-NULL
  opar$cra<-NULL
  opar$csi<-NULL
  opar$cxy<-NULL
  opar$din<-NULL
  opar$page<-NULL
  par(opar)
}
